/**!
 * 
 * K米内置浏览器的Javascript API(离线版)
 * 
 * version :1.0
 * by 2017-11-15
 */

;(function (window) {

    "use strict";

    /**
     * 定义KmApi
     */
    var KmApi = {
        version: 1.0,
        bridge : null,
        isapp : false,
        load : 0,
        data : {}, //全局存储临时数据
        authconifg:{
            "debug":false,
            "webappid": "",
            "webauthtoken": ""
        },
        authtoken:''
    };

    // 将KmApi暴露到window下：全局可使用，对旧版本向下兼容
    window.KmApi = KmApi;

    /////////////////////////// CommonJS /////////////////////////////////

    /**
     * 对象简单继承，后面的覆盖前面的，继承深度：deep=1
     * @private
     */
    var _extend = function () {
        var result = {}, obj, k;
        for (var i = 0, len = arguments.length; i < len; i++) {
            obj = arguments[i];
            if (typeof obj === 'object') {
                for (k in obj) {
                    typeof obj[k]=='boolean'?result[k] = obj[k]:obj[k] && (result[k] = obj[k]);
                }
            }
        }
        return result;
    };
    
    //通用Web打开App页面或者跳转到指定页面
	KmApi.webToAppGotopage = function(data) {

		var defaults = { "webauthtoken":KmApi.authtoken };
		
		var data = _extend(defaults,data);
		var param = new Object(); //参数
		param = {
			'cmd': 'gotopage',
			'data': data
		};

		this.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
      	});
	};
	
	/**
	 * 绑定包厢(APP不校验是否已登录)
	 * @param {Function} callback
	 */
	KmApi.webToAppBindKtv = function(callback){
    	 
      	var param = {
      		'cmd': 'bindktv',
            'data':{"webauthtoken":KmApi.authtoken,"needlogin":"0"}
      	};
      	
      	this.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
          	if(typeof callback === "function"){ callback(response.data); }
      	});
   	};

    /**
     * 解绑包厢
     * @param {Function} callback
     */
    KmApi.webToAppUnbindKtv = function(callback){
         
        var param = {
            'cmd': 'unbindktv',
            'data':{"webauthtoken":KmApi.authtoken,"hinttext":""}
        };
        
        this.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
          	if(typeof callback === "function"){ callback(response.data); }
        });
    };
   	
	//获取包厢信息
	KmApi.getRoomInfo = function(callback){
			
		var param = {
			'cmd': 'getroominfo',
			'data': {"webauthtoken":KmApi.authtoken}
		};
		
		this.bridge.callHandler('WebToLocalAppCallback', param, function(response) {
      		if(typeof callback === "function"){ callback(response.data); }
		});
	};

	//获取当前绑定商家vip配置
	KmApi.getCompanyVipState = function(callback){
		var param = {
			'cmd': 'companyvipstate',
			'data': {"webauthtoken":KmApi.authtoken}
		};
		
		this.bridge.callHandler('WebToLocalAppCallback', param, function(response) {
      		if(typeof callback === "function"){ callback(response.data); }
		});
	};
	
	/**
	 * E协议透传
	 */
	KmApi.eProtocol = function(data,callback){
	 	  var param = {
      		'cmd': 'i18nEprotocol',
          'data':{
            	"webauthtoken":KmApi.authtoken,
        		  "xml":data.xml
        	}
      };

      // alert(param.data.xml);
      this.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
          // alert(JSON.stringify(response));
			    if(typeof callback === 'function'){ callback(response); }
      });
	}

	/**
	 * D协议透传
	 */
	KmApi.dProtocol = function(data,callback){
	 	  var param = {
      		'cmd': 'i18nDprotocol',
          'data':{
            	"webauthtoken":KmApi.authtoken,
        		  "header":data.header,
        		  "body":data.body
        	}
      };

      // alert(JSON.stringify(param.data));
      this.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
        	// alert(JSON.stringify(response));
			   if(typeof callback === 'function'){ callback(response); }
      });
	}

  /**
   * 获取APP语言
   */
   KmApi.getAppLang = function(callback){
      var param = {
          'cmd' : 'i18nLang',
          'data' : {
              "webauthtoken" : KmApi.authtoken
          }
      };
      this.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
          if(typeof callback === 'function'){ callback(response.data); }
      });
  } 

  /**
   * 自定义IOSwebview样式
   */
  KmApi.iOSSafeAreaSetting = function(data){
      var param = {
          'cmd' : 'iOSSafeAreaSetting',
          'data' : {
              "webauthtoken" : KmApi.authtoken
          }
      };

      param.data = _extend(param.data, data);

      this.bridge.callHandler('WebToLocalAppCallback',param , function() {
      });
  }

	/*------------大数据统计3.0---------------*/
	//页面事件上报
	var _bigdatapageevent = function(options){
		var defaults = {
				"type": "0",
				"pagename": "kmweb_"+document.title+"_"+window.location.href,
				"webauthtoken":KmApi.authtoken
			};

        defaults = _extend(data, options);
			
		var param = {'cmd': 'bigdatapageevent','data':defaults};
	    KmApi.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
	    	//alert("pe="+response.data.result);
	    });	
	};
	
	//点击事件上报
	KmApi.bigdataevent = function(options){
		var defaults = {
				"eventid": "kmweb",
				"operatype": "网页点击",
				"operaparam":"",
				"duration":"0",
				"webauthtoken":KmApi.authtoken
			};

        defaults = _extend(data, options);
			
		var param = {'cmd': "bigdataevent",'data':defaults};
	    KmApi.bridge.callHandler('WebToLocalAppCallback',param , function(response) {
	    	//alert("e="+response.data.result);
	    });	
	};

	/**
	 * 加载授权配置文件
	 * @param {Object} options
	 */
    KmApi.config = function(options) {
    	
        var defaults = {
            "debug":false,
            "webappid": "",
            "webauthtoken": ""
        };

       	KmApi.authconifg = _extend(defaults, options);
    };
    
	/**
     * 当页面加载完毕后执行，使用方法：
     * KmApi.ready(function(Api){
     *     // 从这里只用Api即是KmApi
     * });
     * @param {Function} readyCallback
     */
	KmApi.ready = function(readyCallback){

		if(KmApi.load == 1){
			return ;
		}else if(KmApi.load == 0){
			KmApi.load = 1;
		}
		
		if (readyCallback && typeof readyCallback == 'function') {
        	var Api = this;
            var kmReadyFunc = function (event) {
            	
               	KmApi.bridge = event.bridge;
               	KmApi.isapp = true;
				KmApi.load = 2;

				KmApi.bridge.init(function(message, responseCallback) {
			        var data = { 'init':'ok!' };
			    });
			    
			    KmApi.bridge.registerHandler('LocalAppToWebCallback', function(data, responseCallback) {
				});

			    readyCallback(Api);
            };
            
            if ( KmApi.bridge === null) {
                if (document.addEventListener) {
                    document.addEventListener('WebViewJavascriptBridgeReady', kmReadyFunc, true);
                } else if (document.attachEvent) {
                    document.attachEvent('WebViewJavascriptBridgeReady', kmReadyFunc);
                    //document.attachEvent('WebToLocalAppCallback', kmReadyFunc);
                }
            }else{
            	console.log('JsBridge 已经初始化成功...');
			    readyCallback(Api);
            }
        }
	};
	
	//开发初始化接口
	KmApi._openInitBridge = function(event){

		if(KmApi.load == 1){
			return ;
		}else if(KmApi.load == 0){
			KmApi.load = 1;
		}
		
		if(KmApi.bridge === null){

			KmApi.bridge = event.bridge;
			KmApi.isapp = true;
			KmApi.load = 2;

			KmApi.bridge.init(function(message, responseCallback) {
		        var data = { 'init':'ok!' };
		    });

		    KmApi.bridge.registerHandler('LocalAppToWebCallback', function(data, responseCallback) {
			});
		}
	};
	
	/**
     * 开启Api的debug模式，比如出了个什么错误，能alert告诉你
     * @param    {Function}  callback(error) 出错后的回调，默认是alert
     */
    KmApi.enableDebugMode = function (callback) {
        /**
         * @param {String}  errorMessage   错误信息
         * @param {String}  scriptURI      出错的文件
         * @param {Long}    lineNumber     出错代码的行号
         * @param {Long}    columnNumber   出错代码的列号
         */
        window.onerror = function (errorMessage, scriptURI, lineNumber, columnNumber) {
            if(scriptURI.length == 0){
                return;
            }

            // 有callback的情况下，将错误信息传递到options.callback中
            if (typeof callback === 'function') {
                callback({
                    message: errorMessage,
                    script: scriptURI,
                    line: lineNumber,
                    column: columnNumber
                });
            } else {
                 // 其他情况，都以alert方式直接提示错误信息
                var msgs = [];
                msgs.push("\n错误信息：", errorMessage);
                msgs.push("\n出错文件：", scriptURI);
                msgs.push("\n出错位置：", lineNumber + '行，' + columnNumber + '列');
                alert(msgs.join(''));
            }
        }
    };
	
})(window);

document.addEventListener('WebViewJavascriptBridgeReady', openInitBridge, true);
//开发初始化接口
function openInitBridge(event){
	KmApi._openInitBridge(event);
}
